export default {
  name: 'Markers',
  langList: [
    { id: 'nomarker', title: 'No Marker' },
    { id: 'leftarrow', title: 'Left Arrow' },
    { id: 'rightarrow', title: 'Right Arrow' },
    { id: 'textmarker', title: 'Text Marker' },
    { id: 'forwardslash', title: 'Forward Slash' },
    { id: 'reverseslash', title: 'Reverse Slash' },
    { id: 'verticalslash', title: 'Vertical Slash' },
    { id: 'box', title: 'Box' },
    { id: 'star', title: 'Star' },
    { id: 'xmark', title: 'X' },
    { id: 'triangle', title: 'Triangle' },
    { id: 'mcircle', title: 'Circle' },
    { id: 'leftarrow_o', title: 'Open Left Arrow' },
    { id: 'rightarrow_o', title: 'Open Right Arrow' },
    { id: 'box_o', title: 'Open Box' },
    { id: 'star_o', title: 'Open Star' },
    { id: 'triangle_o', title: 'Open Triangle' },
    { id: 'mcircle_o', title: 'Open Circle' }
  ],
  contextTools: [
    {
      title: 'Start marker',
      label: 's'
    },
    {
      title: 'Select start marker type'
    },
    {
      title: 'Middle marker',
      label: 'm'
    },
    {
      title: 'Select mid marker type'
    },
    {
      title: 'End marker',
      label: 'e'
    },
    {
      title: 'Select end marker type'
    }
  ]
}
