export default {
  message: 'By default and where supported, SVG-Edit can store your editor ' +
  'preferences and SVG content locally on your machine so you do not ' +
  'need to add these back each time you load SVG-Edit. If, for privacy ' +
  'reasons, you do not wish to store this information on your machine, ' +
  'you can change away from the default option below.',
  storagePrefsAndContent: 'Store preferences and SVG content locally',
  storagePrefsOnly: 'Only store preferences locally',
  storagePrefs: 'Store preferences locally',
  storageNoPrefsOrContent: 'Do not store my preferences or SVG content locally',
  storageNoPrefs: 'Do not store my preferences locally',
  rememberLabel: 'Remember this choice?',
  rememberTooltip: 'If you choose to opt out of storage while remembering this choice, the URL will change so as to avoid asking again.'
}
