export default {
  message: "Par défaut et si supporté, SVG-Edit peut stocker les préférences de l'éditeur " +
  "et le contenu SVG localement sur votre machine de sorte que vous n'ayez pas besoin de les " +
  'rajouter chaque fois que vous chargez SVG-Edit. Si, pour des raisons de confidentialité, ' +
  'vous ne souhaitez pas stocker ces données sur votre machine, vous pouvez changer ce ' +
  'comportement ci-dessous.',
  storagePrefsAndContent: 'Store preferences and SVG content locally',
  storagePrefsOnly: 'Only store preferences locally',
  storagePrefs: 'Store preferences locally',
  storageNoPrefsOrContent: 'Do not store my preferences or SVG content locally',
  storageNoPrefs: 'Do not store my preferences locally',
  rememberLabel: 'Remember this choice?',
  rememberTooltip: "Si vous choisissez de désactiver le stockage en mémorisant le choix, l'URL va changer afin que la question ne vous soit plus reposée."
}
