export default {
  message: 'Standardmäßig kann SVG-Edit Ihre Editor-Einstellungen ' +
  'und die SVG-Inhalte lokal auf Ihrem Gerät abspeichern. So brauchen Sie ' +
  'nicht jedes Mal die SVG neu laden. Falls Sie aus Datenschutzgründen ' +
  'dies nicht wollen, ' +
  'können Sie die Standardeinstellung im Folgenden ändern.',
  storagePrefsAndContent: 'Store preferences and SVG content locally',
  storagePrefsOnly: 'Only store preferences locally',
  storagePrefs: 'Store preferences locally',
  storageNoPrefsOrContent: 'Do not store my preferences or SVG content locally',
  storageNoPrefs: 'Do not store my preferences locally',
  rememberLabel: 'Remember this choice?',
  rememberTooltip: 'If you choose to opt out of storage while remembering this choice, the URL will change so as to avoid asking again.'
}
