export default {
  name: 'İşaretleyiciler',
  langList: [
    { id: 'nomarker', title: 'İşaretleyici Yok' },
    { id: 'leftarrow', title: 'Sol Ok' },
    { id: 'rightarrow', title: 'Sağ Ok' },
    { id: 'textmarker', title: 'Metin İşaretleyici' },
    { id: 'forwardslash', title: 'İleri Bölme İşareti' },
    { id: 'reverseslash', title: 'Geri Bölme İşareti' },
    { id: 'verticalslash', title: 'Dikey Bölme İşareti' },
    { id: 'box', title: 'Kutu' },
    { id: 'star', title: 'Yıldız' },
    { id: 'xmark', title: 'X' },
    { id: 'triangle', title: 'Üçgen' },
    { id: 'mcircle', title: 'Çember' },
    { id: 'leftarrow_o', title: 'Sola Açık Ok' },
    { id: 'rightarrow_o', title: 'Sağa Açık Ok' },
    { id: 'box_o', title: 'Açık Kutu' },
    { id: 'star_o', title: 'Açık Yıldız' },
    { id: 'triangle_o', title: 'Açık Üçgen' },
    { id: 'mcircle_o', title: 'Açık Çember' }
  ],
  contextTools: [
    {
      title: 'İşaretleyici Başla',
      label: 's'
    },
    {
      title: 'Başlangıç işaretleyici seçin'
    },
    {
      title: 'Orta İşaretleyici',
      label: 'm'
    },
    {
      title: 'Orta işaretleyici seçin'
    },
    {
      title: 'Bitiş işaretleyicisi',
      label: 'e'
    },
    {
      title: 'Bitiş işaretleyicisi seçin'
    }
  ]
}
