export default {
  message: 'Varsayılan olarak ve desklendiğinde, SVG-Edit ayarlarlarınızı ve ' +
  'SVG\'lerinizi yerel makinenizde kaydedebilir dolayısıyla tekrar ' +
  'SVG-Edit\'i her açtığınızda tekrar yüklemeniz gerekmez.Eğer, gizlilik gereği ' +
  'bu bilgileri bilgisayarınızda saklamak istemiyorsanız, ' +
  'varsayılan ayarı buradan değiştirebilirsiniz.',
  storagePrefsAndContent: 'Ayarları ve SVG içeriği Yerel Olarak Sakla',
  storagePrefsOnly: 'Yalnızda ayarları yerel olarak sakla',
  storagePrefs: 'Arayları Yerel Olarak Saklayın',
  storageNoPrefsOrContent: 'Ayarlarımı ya da SVG içeriği yerel olarak saklama!',
  storageNoPrefs: 'Ayarlarımı yerel olarak saklama!',
  rememberLabel: 'Bu seçim hatırlansın mı?',
  rememberTooltip: 'Yerel saklama ayarını kapatırsanız, aynı soruyu tekrar sormamak için URL değişecek.'
}
