export default {
  message: '默认情况下, SVG-Edit 在本地保存配置参数和画布内容. 如果基于隐私考虑, ' +
  '您可以勾选以下选项修改配置.',
  storagePrefsAndContent: '本地存储配置参数和SVG图',
  storagePrefsOnly: '本地只存储配置参数',
  storagePrefs: '本地存储配置参数',
  storageNoPrefsOrContent: '本地不保存配置参数和SVG图',
  storageNoPrefs: '本地不保存配置参数',
  rememberLabel: '记住选择?',
  rememberTooltip: '如果您勾选记住选择，将不再弹出本窗口.'
}
